package com.as.common.jwt;

import java.io.PrintWriter;
import java.lang.reflect.Method;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import com.as.common.json.requestJSON;
import com.as.user.dao.UserData;
import com.as.user.service.UserService;
import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.JWTDecodeException;
import com.auth0.jwt.exceptions.JWTVerificationException;

import cn.hutool.json.JSONUtil;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;

/**
 * Date:2019/11/29
 * 拦截器去获取token并验证token
 * @author:lg
 */
public class AuthenticationInterceptor implements HandlerInterceptor {
    @Autowired
    private UserService userService;
    @Override
    public boolean preHandle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o) throws Exception {
    	requestJSON<String> requestJSON = new requestJSON<String>();
        //从请求头中取出token
        String token  = httpServletRequest.getHeader("Authorization");
        //如果不是直接映射到方法就直接通过
        if(!(o instanceof HandlerMethod)){
            return true;
        }
        HandlerMethod handlerMethod = (HandlerMethod) o;
        Method method = handlerMethod.getMethod();
        //检查方法上是否有自定义的注解，有则放行
        if (method.isAnnotationPresent(PassToken.class)){
            PassToken annotation = method.getAnnotation(PassToken.class);
            if (annotation.required()){
                return true;
            }
        }

        //检查有没有需要用户权限的注解
        if (method.isAnnotationPresent(UserLoginToken.class)){
            UserLoginToken loginToken = method.getAnnotation(UserLoginToken.class);
            if (loginToken.required()){
                if (token == null){
                	requestJSON.setCode(500);
                	requestJSON.setMessage("无token ，请重新登录");
                	httpServletResponse.setHeader("Access-Control-Allow-Origin", "*");
                	httpServletResponse.setHeader("Cache-Control", "no-cache");
                	httpServletResponse.setCharacterEncoding("UTF-8");
                	httpServletResponse.setContentType("application/json");
                	PrintWriter out = httpServletResponse.getWriter();
                	out.write(JSONUtil.toJsonStr(requestJSON));
                	out.flush();
            		out.close();
//                    throw new RuntimeException("无token ，请重新登录");
            		return false;
                }
                String userId = "";
                try {
                    userId = JWT.decode(token).getAudience().get(0);
                } catch (JWTDecodeException e) {
                    e.printStackTrace();
                }
//                System.out.println("userId" + userId);
                UserData userByUserId = userService.findUserByUserId(Integer.parseInt(userId));
                System.out.println("userByUserId" + userByUserId);
                if (userByUserId == null){
                	requestJSON.setCode(500);
                	requestJSON.setMessage("用户不存在 ，请重新登录");
                	httpServletResponse.setHeader("Access-Control-Allow-Origin", "*");
                	httpServletResponse.setHeader("Cache-Control", "no-cache");
                	httpServletResponse.setCharacterEncoding("UTF-8");
                	httpServletResponse.setContentType("application/json");
                	PrintWriter out = httpServletResponse.getWriter();
                	out.write(JSONUtil.toJsonStr(requestJSON));
                	out.flush();
            		out.close();
//                    throw new RuntimeException("无token ，请重新登录");
            		return false;
                }
                
                //验证token

                JWTVerifier jwtVerifier = JWT.require(Algorithm.HMAC256(userByUserId.getPwd())).build();
                try {
                    jwtVerifier.verify(token);
                } catch (JWTVerificationException e) {
                    throw new RuntimeException("401");
                }
                return  true;
            }
        }
        //没有相关注解的全部放行，
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o, ModelAndView modelAndView) throws Exception {

    }

    @Override
    public void afterCompletion(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o, Exception e) throws Exception {

    }
}


