package com.as.workflow.service.impl;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

import com.as.common.exception.base.BaseException;
import com.as.common.utils.StringUtils;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.as.user.dao.Role;
import com.as.user.dao.UserData;
import com.as.workflow.dao.ViewWorkFlowEntity;
import com.as.workflow.dao.WorkFlowEntity;
import com.as.workflow.mapper.ViewWorkFlowMapper;
import com.as.workflow.mapper.WorkFlowMapper;
import com.as.workflow.service.WorkFlowService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
@Service
public class WorkFlowServiceImpl implements WorkFlowService {
	 	@Autowired
	    private ViewWorkFlowMapper viewWorkFlowMapper;
	 	@Autowired
	    private WorkFlowMapper workFlowMapper;
	 	
		/**
 		 * 查询所有状态的数量
 		 */
	 	@Override
	 	public List<HashMap<String,Object>> getStatus(WorkFlowEntity workFlowEntity){
	 		HashMap<String, Object> hashMapOne = new HashMap<String,Object>();
	 		HashMap<String, Object> hashMapTwo = new HashMap<String,Object>();
	 		List<HashMap<String, Object>> list = new ArrayList<HashMap<String,Object>>();
	 		LambdaQueryWrapper<ViewWorkFlowEntity> queryWrapper = new LambdaQueryWrapper<>();
	 		queryWrapper.like(StringUtils.isNotEmpty(workFlowEntity.getWorkOrderNo()), ViewWorkFlowEntity::getWorkOrderNo, workFlowEntity.getWorkOrderNo());
			queryWrapper.eq(StringUtils.isNotEmpty(workFlowEntity.getWorkPlace()), ViewWorkFlowEntity::getWorkPlace, workFlowEntity.getWorkPlace());
			queryWrapper.like(StringUtils.isNotEmpty(workFlowEntity.getPatientName()), ViewWorkFlowEntity::getPatientName, workFlowEntity.getPatientName());
			
			queryWrapper.ge(StringUtils.isNotEmpty(workFlowEntity.getCreatedStartDt()) , ViewWorkFlowEntity::getCreatedDt, workFlowEntity.getCreatedStartDt()); // 大于等于
			queryWrapper.le(StringUtils.isNotEmpty(workFlowEntity.getCreatedEndDt()), ViewWorkFlowEntity::getCreatedDt, workFlowEntity.getCreatedEndDt());// 小于等于
			
			hashMapTwo.put("value", viewWorkFlowMapper.selectCount(queryWrapper));
			hashMapOne.put("name", "全部");
			hashMapOne.put("badge", hashMapTwo);
			list.add(hashMapOne);
			list.add(makeMap("工单开始",workFlowEntity));
			list.add(makeMap("取未煎药",workFlowEntity));
			list.add(makeMap("未煎药送达",workFlowEntity));
			list.add(makeMap("煎药完成",workFlowEntity));
			list.add(makeMap("取已煎药",workFlowEntity));
			list.add(makeMap("完成",workFlowEntity));
			
	 		return list;
	 	};
	 	
	 	public HashMap<String, Object> makeMap(String status, WorkFlowEntity workFlowEntity){
	 		HashMap<String, Object> hashMapOne = new HashMap<String,Object>();
	 		HashMap<String, Object> hashMapTwo = new HashMap<String,Object>();
	 		LambdaQueryWrapper<ViewWorkFlowEntity> queryWrapper = new LambdaQueryWrapper<>();
	 		queryWrapper.like(StringUtils.isNotEmpty(workFlowEntity.getWorkOrderNo()), ViewWorkFlowEntity::getWorkOrderNo, workFlowEntity.getWorkOrderNo());
			queryWrapper.eq(StringUtils.isNotEmpty(workFlowEntity.getWorkPlace()), ViewWorkFlowEntity::getWorkPlace, workFlowEntity.getWorkPlace());
			queryWrapper.like(StringUtils.isNotEmpty(workFlowEntity.getPatientName()), ViewWorkFlowEntity::getPatientName, workFlowEntity.getPatientName());
			
			queryWrapper.ge(StringUtils.isNotEmpty(workFlowEntity.getCreatedStartDt()) , ViewWorkFlowEntity::getCreatedDt, workFlowEntity.getCreatedStartDt()); // 大于等于
			queryWrapper.le(StringUtils.isNotEmpty(workFlowEntity.getCreatedEndDt()), ViewWorkFlowEntity::getCreatedDt, workFlowEntity.getCreatedEndDt());// 小于等于
			queryWrapper.eq(ViewWorkFlowEntity::getStatus, status);
			hashMapTwo.put("value", viewWorkFlowMapper.selectCount(queryWrapper));
			hashMapOne.put("name", status);
			hashMapOne.put("badge", hashMapTwo);
			
			return hashMapOne;
	 	};
	 	
	 	/**
	 	 * 按照工作流id查询本id的数据
	 	 */
		@Override
	 	public List<WorkFlowEntity> findWorkFlowList(WorkFlowEntity workFlowEntity) throws Exception{
			LambdaQueryWrapper<WorkFlowEntity> qw = new LambdaQueryWrapper<>();
			qw.eq(workFlowEntity.getWorkOrderNo() != null, WorkFlowEntity::getWorkOrderNo, workFlowEntity.getWorkOrderNo());
			List<WorkFlowEntity> list = workFlowMapper.selectList(qw);
			return list;
	 	}
		/**
		 * 带有分页的视图查询，通过倒序查询到以工单id分组的数据。视图名为select_as_physical_distribution_group_by_work_order_no
		 */
		@Override
	 	public  HashMap<String, Object> findViewToWokeList(WorkFlowEntity workFlowEntity) throws Exception{
			LambdaQueryWrapper<ViewWorkFlowEntity> queryWrapper = new LambdaQueryWrapper<>();
			queryWrapper.like(StringUtils.isNotEmpty(workFlowEntity.getWorkOrderNo()), ViewWorkFlowEntity::getWorkOrderNo, workFlowEntity.getWorkOrderNo());
			queryWrapper.eq(StringUtils.isNotEmpty(workFlowEntity.getWorkPlace()), ViewWorkFlowEntity::getWorkPlace, workFlowEntity.getWorkPlace());
			queryWrapper.like(StringUtils.isNotEmpty(workFlowEntity.getPatientName()), ViewWorkFlowEntity::getPatientName, workFlowEntity.getPatientName());
			queryWrapper.eq(StringUtils.isNotEmpty(workFlowEntity.getStatus()), ViewWorkFlowEntity::getStatus, workFlowEntity.getStatus());
			queryWrapper.ge(StringUtils.isNotEmpty(workFlowEntity.getCreatedStartDt()) , ViewWorkFlowEntity::getCreatedDt, workFlowEntity.getCreatedStartDt()); // 大于等于
			queryWrapper.le(StringUtils.isNotEmpty(workFlowEntity.getCreatedEndDt()), ViewWorkFlowEntity::getCreatedDt, workFlowEntity.getCreatedEndDt());// 小于等于
			queryWrapper.orderByDesc(ViewWorkFlowEntity::getSid);
			Page<ViewWorkFlowEntity> page = new Page<>(workFlowEntity.getPageNum(), workFlowEntity.getPageSize());
			viewWorkFlowMapper.selectPage(page, queryWrapper);
			HashMap<String, Object> hashMap = new HashMap<String, Object>();
			List<ViewWorkFlowEntity> list = page.getRecords();
			Long current = page.getTotal();
			hashMap.put("count", current);
			hashMap.put("data", list);
			return hashMap;
		}

		/**
		 * 新增方法，若没有传过来工单号的情况下需要生成工单号。如果传过来工单号，则需要新增的内容中增加工单的内容
		 * @throws Exception
		 */
		@Override
	 	public Integer addWorkFlow(WorkFlowEntity workFlowEntity,List<Role> roleList) throws Exception{
			// TODO 需要鉴权，根据操作角色查询本条信息是否应该本角色操作（）
			// 
			
			if(workFlowEntity.getWorkOrderNo()==null) {
				workFlowEntity.setWorkOrderNo(makeWorkFlowOrderId());
			}

			// 查询本工单最后一个状态，然后通过权限对比，决定是否新增
			return workFlowMapper.insert(workFlowEntity);
		}

		private String makeWorkFlowOrderId() throws Exception{

			// 查询id最大的那个订单号 然后截取最后四位，将其转换成数字，然后给这个数字加1
			QueryWrapper<WorkFlowEntity> queryWrapper = new QueryWrapper<>();
			queryWrapper.orderByDesc("sid");
			queryWrapper.ge("created_dt",new Date());
			Page<WorkFlowEntity> page = new Page<>(1, 1);
			workFlowMapper.selectPage(page, queryWrapper);
			List<WorkFlowEntity> list = page.getRecords();
			// 获取当前日期并格式化为YYYYmmdd
			SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
			String currentDate = sdf.format(new Date());
			if(list.size()>0) {
				String	str = list.get(0).getWorkOrderNo();
				// 截取最后四位
				// 然后返回AS + 当前YYYYmmdd + 刚才的数字
				String lastFourDigits = str.substring(str.length() - 4);
				// 转换为整数并加1
				int number = Integer.parseInt(lastFourDigits) + 1;
				String formattedNumber = String.format("%04d", number);
				System.out.println(formattedNumber);

				// 生成最终的字符串
				String finalString = "AS" + currentDate + formattedNumber;
				return finalString;
			}else {
				return 	"AS" + currentDate + "0001";

			}
		}
}
